/// <reference types="node" />
import { EventEmitter } from "events";
import puppeteer from "puppeteer";
import { Device, Viewport } from "puppeteer/DeviceDescriptors";
import { DeviceNames } from "./devices";
import { ViewportInput } from "./utils/get-viewport";
declare type DeviceOrViewport = Device | Viewport;
declare type OutputOptions = puppeteer.Base64ScreenShotOptions | puppeteer.PDFOptions;
declare type CaptureType = "buffer" | "base64" | "html" | "pdf" | "jpeg" | "png";
/**
 * A set of options used to customize the request
 */
interface RequestOptions {
    /**
     * Headers to be added on request
     */
    headers?: {
        [header: string]: string;
    };
    /**
     * The request timeout
     */
    timeout?: number;
    /**
     * An array of scripts to add
     */
    scripts?: any[];
    /**
     * An array of styles to add
     */
    styles?: any[];
    /**
     * Set the event that page will wait
     */
    waitUntil?: puppeteer.LoadEvent;
}
/**
 * Options used to customize the captured output
 * and the capture execution such as viewports, headers and more
 */
interface CaptureOptions extends RequestOptions {
    /**
     * The output format type
     */
    type?: CaptureType;
    /**
     * Viewport to capture
     */
    viewport?: ViewportInput | ViewportInput[];
    /**
     * A category of viewports to capture
     */
    viewportCategory?: string;
    /**
     * A string selector to capture on page
     */
    selector?: string;
    /**
     * Waits for a defined time or selector
     */
    waitFor?: number | string;
    /**
     * Additional capture options passed to puppeteer
     */
    options?: OutputOptions;
}
interface InstanceOptions extends Pick<Options, Exclude<keyof Options, "viewport">> {
    viewport: DeviceOrViewport;
}
interface FilenameOptions {
    input?: any;
    name?: string;
    viewport?: string;
    type?: string;
}
/**
 * The capture result object containing informations
 * about the capture output and the execution process
 */
interface Result {
    input: string;
    output: any;
    duration: number;
    error?: string;
}
/**
 * Capturer constructor options
 * used to customize browser instance
 * and set the default request properties
 * such as headers, default viewport and more
 */
export interface Options extends RequestOptions {
    /**
     * Enable the debug headfull mode
     */
    debug?: boolean;
    /**
     * Path to a custom output directory
     */
    outputDir?: string;
    /**
     * Additional arguments to pass to the browser instance.
     * The list of Chromium flags can be found here.
     */
    launchArgs?: string[];
    /**
     * Default timeout value for requests
     */
    timeout?: number;
    /**
     * The default viewport to use in case not specified
     */
    viewport?: DeviceNames | puppeteer.Viewport;
}
/**
 * The capturer class that will run with customised settings
 * such as custom output path, default headers, default viewport and more
 */
export declare class WebpageCapture extends EventEmitter {
    options: InstanceOptions;
    page: puppeteer.Page;
    private browser;
    private counter;
    private styles;
    private scripts;
    constructor(opts?: Options);
    /**
     * Prepare the browser instance, spawn a new page
     * and set the default viewport and configurations
     */
    prepare(): Promise<void>;
    /**
     * Return a base64 encoded string for a given source
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    base64(source: string, opts?: Partial<CaptureOptions>): Promise<Result>;
    /**
     * Return a binary string buffer for a given source
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    buffer(source: string, opts?: Partial<CaptureOptions>): Promise<Result>;
    /**
     * Capture the source to file and return the output path
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    file(source: string, output: string, opts?: Partial<CaptureOptions>): Promise<Result>;
    /**
     * Prepare the sources and loop through each of them
     * running the capture function with defined options
     * return the results array with capture metadata
     */
    capture(sources: string | string[], opts?: CaptureOptions): Promise<Result[]>;
    /**
     * Reset all the variables and closes the browser instance
     */
    close(): Promise<this>;
    /**
     * Construct the file output path from the given input
     * which can be a url, a string with name or something else
     * or an object with name property, otherwise create a custom name
     * based on execution counter and current timestamp
     */
    getOutPath({ input, name, viewport, type, }: FilenameOptions): string;
    /**
     * Prepare the page by visiting the url or loading the html content
     * than it optionally load additional script and styles
     */
    private preparePage;
    /**
     * Takes the actual screenshot or get the page HTML content
     */
    private render;
    /**
     * For a given source and capture options
     * render the output for each desired viewport and settings combination
     */
    private run;
}
export {};
