"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const perf_hooks_1 = require("perf_hooks");
const url_1 = __importDefault(require("url"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const config_1 = require("./config");
const get_viewport_1 = require("./utils/get-viewport");
const prepare_arguments_1 = __importDefault(require("./utils/prepare-arguments"));
const prepare_script_options_1 = __importDefault(require("./utils/prepare-script-options"));
const prepare_viewports_1 = __importDefault(require("./utils/prepare-viewports"));
const type_guards_1 = require("./utils/type-guards");
const validators_1 = require("./utils/validators");
/**
 * The capturer class that will run with customised settings
 * such as custom output path, default headers, default viewport and more
 */
class WebpageCapture extends events_1.EventEmitter {
    constructor(opts = {}) {
        super();
        this.counter = 0;
        this.styles = [];
        this.scripts = [];
        // Set internal options
        this.options = Object.assign({}, config_1.defaultOptions, opts, { viewport: config_1.defaultViewport });
        if (opts.viewport) {
            const viewport = get_viewport_1.getViewport(opts.viewport);
            if (!viewport) {
                throw new Error(`Viewport "${opts.viewport}" is not supported.`);
            }
            this.options.viewport = viewport;
        }
        // Ensure the output folder exists or fail
        mkdirp_1.default.sync(this.options.outputDir);
    }
    /**
     * Prepare the browser instance, spawn a new page
     * and set the default viewport and configurations
     */
    prepare() {
        return __awaiter(this, void 0, void 0, function* () {
            const { debug, launchArgs } = this.options;
            const launchOpts = {
                headless: !debug,
                slowMo: debug ? 1000 : 0,
                args: launchArgs,
            };
            // Launch the browser instance
            this.browser = yield puppeteer_1.default.launch(launchOpts);
            // Get the default about:blank page
            this.page = (yield this.browser.pages())[0];
            // Optionally set the default headers
            if (this.options.headers) {
                yield this.page.setExtraHTTPHeaders(this.options.headers);
            }
            // Optionally override the default navigation timeout
            if (this.options.timeout) {
                this.page.setDefaultNavigationTimeout(this.options.timeout);
            }
            // Set the page viewport
            if (type_guards_1.isDevice(this.options.viewport)) {
                yield this.page.emulate(this.options.viewport);
            }
            else {
                yield this.page.setViewport(this.options.viewport);
            }
        });
    }
    /**
     * Return a base64 encoded string for a given source
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    base64(source, opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const [res] = yield this.capture(source, lodash_1.default.merge(opts, {
                type: "base64",
            }));
            return res;
        });
    }
    /**
     * Return a binary string buffer for a given source
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    buffer(source, opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const [res] = yield this.capture(source, lodash_1.default.merge(opts, {
                type: "buffer",
            }));
            return res;
        });
    }
    /**
     * Capture the source to file and return the output path
     * it calls capture method internally defaulting some options
     * and it return a single element instead of an array of results
     */
    file(source, output, opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const ext = path_1.default.extname(output).substr(1);
            const [res] = yield this.capture(source, lodash_1.default.merge(opts, {
                type: ext || "png",
                name: output,
            }));
            return res;
        });
    }
    /**
     * Prepare the sources and loop through each of them
     * running the capture function with defined options
     * return the results array with capture metadata
     */
    capture(sources, opts = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            // Ensure default options
            const options = lodash_1.default.merge({
                type: "png",
                options: {},
            }, opts);
            // TODO: Validate execution options
            validators_1.validateType(options.type);
            if (!this.browser) {
                yield this.prepare();
            }
            if (lodash_1.default.isEmpty(sources)) {
                return [];
            }
            // Get the sources to capture
            sources = lodash_1.default.isArray(sources) ? lodash_1.default.uniq(sources) : [sources];
            sources = prepare_arguments_1.default(sources);
            const results = [];
            // Loop each source, load it and capture it in various formats
            for (let i = 0; i < sources.length; i++) {
                const current = i + 1;
                const source = sources[i];
                // TODO: Check if source is url or html content
                const result = {
                    input: source,
                    output: null,
                };
                // Emit the capture start event with current array element
                const startTime = perf_hooks_1.performance.now();
                this.emit("capture:start", Object.assign({}, result, { total: sources.length, current, remaining: sources.length - current }));
                try {
                    const outPath = yield this.run(source, options);
                    result.output = outPath;
                }
                catch (e) {
                    result.error = e.toString();
                    this.emit("capture:error", e);
                }
                // Add the capture duration to the result properties
                result.duration = perf_hooks_1.performance.now() - startTime;
                // Emit the capture end event with the current element
                this.emit("capture:end", Object.assign({}, result, { total: sources.length, current, remaining: sources.length - current }));
                // Increase the global captures counter
                this.counter++;
                // Add the element to the results array
                results.push(result);
            }
            return results;
        });
    }
    /**
     * Reset all the variables and closes the browser instance
     */
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.browser) {
                return this;
            }
            yield this.browser.close();
            this.browser = null;
            this.page = null;
            return this;
        });
    }
    /**
     * Construct the file output path from the given input
     * which can be a url, a string with name or something else
     * or an object with name property, otherwise create a custom name
     * based on execution counter and current timestamp
     */
    getOutPath({ input, name, viewport, type, }) {
        if (name && name !== "") {
            const ext = path_1.default.extname(name);
            if (lodash_1.default.isEmpty(ext)) {
                name = `${name}.${type}`;
            }
            return path_1.default.resolve(name);
        }
        const parts = [];
        const parsedUrl = url_1.default.parse(input || "");
        // TODO: Add support for custom output template string
        if (!parsedUrl.hostname || parsedUrl.hostname === "") {
            parts.push(String(this.counter).padStart(4, "0"));
        }
        else {
            parts.push(parsedUrl.hostname);
        }
        parts.push(viewport);
        parts.push(Date.now());
        return path_1.default.resolve(this.options.outputDir, `${parts.filter((p) => p).join("-")}.${type}`);
    }
    /**
     * Prepare the page by visiting the url or loading the html content
     * than it optionally load additional script and styles
     */
    preparePage(source, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { viewport, scripts, styles, } = options;
            const loadOptions = {
                waitUntil: options.waitUntil || this.options.waitUntil || "load",
            };
            // If viewport exists emulate it
            if (viewport && !lodash_1.default.isArray(viewport)) {
                if (type_guards_1.isDevice(viewport)) {
                    yield this.page.emulate(viewport);
                }
                else {
                    yield this.page.setViewport(viewport);
                }
            }
            // Load the source into page
            if (validators_1.isValidHTML(source)) {
                yield this.page.setContent(source, loadOptions);
            }
            else {
                yield this.page.goto(source, loadOptions);
            }
            // Optionally apply styles
            if (lodash_1.default.isArray(styles)) {
                for (const style of styles) {
                    const opts = prepare_script_options_1.default(style);
                    yield this.page.addStyleTag(opts);
                }
            }
            // Optionally apply scripts
            if (lodash_1.default.isArray(scripts)) {
                for (const script of scripts) {
                    const opts = prepare_script_options_1.default(script);
                    yield this.page.addScriptTag(opts);
                }
            }
        });
    }
    /**
     * Takes the actual screenshot or get the page HTML content
     */
    render(source, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.preparePage(source, options);
            // Optionally wait for a selector or certain amount of time
            if (options.waitUntil || options.waitFor) {
                const waitCondition = options.waitUntil || options.waitFor;
                if (typeof waitCondition === "number") {
                    yield this.page.waitFor(waitCondition);
                }
                else {
                    yield this.page.waitFor(waitCondition, {
                        timeout: this.options.timeout,
                    });
                }
            }
            // Get the output file path based on options
            const viewportName = options.viewport && type_guards_1.isDevice(options.viewport) && options.viewport.name;
            let output = this.getOutPath(Object.assign({}, options, { viewport: viewportName, input: source }));
            // Select a specific element or default to whole page
            const element = options.selector ?
                yield this.page.$(options.selector) :
                this.page;
            // Decide how to capture based on options
            switch (options.type) {
                case "buffer": {
                    output = yield element.screenshot(Object.assign({}, options.options, { encoding: "binary" }));
                    break;
                }
                case "base64": {
                    output = yield element.screenshot(Object.assign({}, options.options, { type: "png", encoding: "base64" }));
                    break;
                }
                case "html": {
                    const htmlContent = yield this.page.content();
                    fs_1.default.writeFileSync(output, htmlContent, "utf8");
                    break;
                }
                case "pdf": {
                    const opts = lodash_1.default.assign({
                        format: "A4",
                    }, options.options);
                    yield this.page.pdf(Object.assign({}, opts, { path: output }));
                    break;
                }
                case "jpeg":
                case "png": {
                    const opts = lodash_1.default.assign({
                        type: options.type,
                    }, options.options);
                    yield element.screenshot(Object.assign({}, opts, { path: output }));
                    break;
                }
                default:
                    throw new Error(`Invalid type option value: ${options.type}`);
            }
            return output;
        });
    }
    /**
     * For a given source and capture options
     * render the output for each desired viewport and settings combination
     */
    run(source, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let viewports;
            if (options.viewportCategory) {
                viewports = prepare_viewports_1.default(options.viewportCategory);
            }
            else {
                if (lodash_1.default.isArray(options.viewport)) {
                    viewports = options.viewport.map((v) => {
                        const viewport = get_viewport_1.getViewport(v);
                        if (lodash_1.default.isUndefined(viewport)) {
                            throw new Error(`Invalid viewport "${v}"`);
                        }
                        return viewport;
                    });
                }
                else if (lodash_1.default.isString(options.viewport)) {
                    const viewport = get_viewport_1.getViewport(options.viewport);
                    if (lodash_1.default.isUndefined(viewport)) {
                        throw new Error(`Invalid viewport "${options.viewport}"`);
                    }
                    viewports = [viewport];
                }
            }
            if (lodash_1.default.isEmpty(viewports)) {
                return this.render(source, Object.assign({}, options, { viewport: null }));
            }
            const results = [];
            for (const viewport of viewports) {
                const out = yield this.render(source, Object.assign({}, options, { viewport }));
                results.push(out);
            }
            return results.length === 1 ? results[0] : results;
        });
    }
}
exports.WebpageCapture = WebpageCapture;
